//RGBEPARTICLES.H
//Particle system of RGBEngine
//
//RGBEngine is an Open Source Project, if you want to use the code
//or parts of it in any of your own projects, please tell it to me
// {webmaster@msaemtech32.zzn.com}
//and give me credit if you release it.
//
//'02, Matvej Soloviev ("blackhole89")

#include <malloc.h>

#ifndef RGBEPARTICLES
#define RGBEPARTICLES

typedef struct tagTDoubleXYInfo {
	double x;
	double y;
} TDoubleXYInfo;

typedef struct tagTParticle {
	TDoubleXYInfo	position;
	double			velx;
	double			vely;
	unsigned long	user_data;	//32 bits for user defined information
} TParticle;

typedef class CParticleSet {
protected:
	TParticle*		particles;
	int				nParticles;
	TDoubleXYInfo	xyOrigin;
	double			nEffectiveGravityPerTurn;	
public:
	CParticleSet() {}
	CParticleSet(double xOrigin, double yOrigin);
	CParticleSet(double xOrigin, double yOrigin, int nParticles, bool bRandom);
	void CreateRandomParticles();
	void DoParticleStep();
	virtual void DrawParticle(int nParticle)=0;

	void SetMaxParticles(int nMaxParticles) {nParticles=nMaxParticles;}
	void	SetEffectiveGPT(double nEGPT)	{nEffectiveGravityPerTurn=nEGPT;}
	double	GetEffectiveGPT()				{return nEffectiveGravityPerTurn;}
	void	StoreParticleUserDataFlag(int npart,int nbit,bool nvalue)
		{
			particles[npart].user_data |= ((2^nbit)*(int)nvalue);
		}
	bool	GetParticleUserDataFlag(int npart,int nbit)
		{
			return particles[npart].user_data & (2^nbit);
		}
	void	SetOriginXY(int newx,int newy) {xyOrigin.x=newx; xyOrigin.y=newy;}
} CParticleSet, *LPPARTICLESET;

CParticleSet::CParticleSet(double xOrigin, double yOrigin, int nMaxParticles, bool bRandomDirection)
{
	xyOrigin.x=xOrigin;
	xyOrigin.y=yOrigin;
	particles=(TParticle*)malloc(nMaxParticles*sizeof(TParticle));
	nParticles=nMaxParticles;
	nEffectiveGravityPerTurn=0.1;
	if(bRandomDirection)
	{
		CreateRandomParticles();
	}
}

CParticleSet::CParticleSet(double xOrigin, double yOrigin)
{
	xyOrigin.x=xOrigin;
	xyOrigin.y=yOrigin;
	nEffectiveGravityPerTurn=0.1;
}

void CParticleSet::CreateRandomParticles()
{
	for(int i=0;i<nParticles;++i)
	{
		particles[i].position.x=0;
		particles[i].position.y=0;
		particles[i].user_data=0;
		particles[i].velx=(double)((double)rand()/(double)RAND_MAX)*5-2.5;
		particles[i].vely=-(double)((double)rand()/(double)RAND_MAX)*3;
	}
}

void CParticleSet::DoParticleStep()
{
	for(int i=0;i<nParticles;++i)
	{
		if(particles[i].position.y<1000)		//Assert & do not calculate >1000
		{
			particles[i].position.x+=particles[i].velx;
			particles[i].position.y+=particles[i].vely;
			particles[i].vely+=nEffectiveGravityPerTurn;
			DrawParticle(i);
		}
	}
}

#endif

//Sample system,	lpParticle is an IDirectDrawSurface (or also IDirectDrawSurface7 etc.)
//					with the picture of a SINGLE particle
//					lpBack is the backbuffer.
LPDDSURF lpParticle;
LPDDSURF lpBack;

class CCTDIIParticleSet : public CParticleSet
{
public:
	CCTDIIParticleSet(double xOrigin, double yOrigin);
	CCTDIIParticleSet(double xOrigin, double yOrigin, int nParticles, bool bRandom);
	bool is_active;
	virtual void DrawParticle(int nParticle)
	{
		lpBack->BltFast((int)particles[nParticle].position.x+xyOrigin.x,(int)particles[nParticle].position.y+xyOrigin.y,lpParticle,NULL,DDBLTFAST_WAIT|DDBLTFAST_SRCCOLORKEY);
	}
};

CCTDIIParticleSet::CCTDIIParticleSet(double xOrigin, double yOrigin)
		:CParticleSet(xOrigin,yOrigin)
{
	is_active=false;
	//Insert a code here f.E. for loading lpParticle
}
CCTDIIParticleSet::CCTDIIParticleSet(double xOrigin, double yOrigin, int nParticles, bool bRandom) 
		:CParticleSet(xOrigin,yOrigin,nParticles,bRandom)
{
	is_active=false;
}